# Program: zip_backup.pl
# Author:  Larry Bednar, 3226 NE 24th Ave., Portland OR 97212
#          503-493-8542
#
# Purpose: This Perl program is designed to execute a backup
#          of files matching a naming pattern and directory 
#          location specified in the setup section of the 
#          program. The program is targeted for MS-DOS/Windows
#          computer systems. 
#
#          Copies of files matching this description  
#          are placed in a WinZip archive named using a base name
#          pattern specified in the program (changeable by the
#          individual setting up the program for operation) and 
#          suffixed with the date of the backup.  The intent is to 
#          provide easy, narrowly focused backup abilities that can 
#          be accessed via a desktop shortcut in Windows. The 
#          resulting backup is not mistakable as the original copy 
#          of the files backed up since it is stored in a WinZip
#          archive file. 
#
# Details: The program expects Perl and WinZip 8.1 (with command
#          line add-on) to be installed on the machine from
#          which it is run.
#
#          The user may customize the program readily in seven
#          places - all marked by a "# USER MODIFICATION" comment
#          statement immediately preceding the assignment statement
#          that might be modified.  These modifications affect  
#          the values specifying the source, target, and WinZip
#          installation directories, the base name of backup archive
#          files, the name of the backup history file, the name
#          patterns of files to be included in the backup, and 
#          the abbreviation used to indication the operating system
#          under which the program will be run.  It is advisable to
#          modify a copy, and retain an unaltered version of the
#          program as originally provided.
#
#          Due to use of the "back slash" character for special 
#          purposes in Perl, the user will have to be sure to
#          specify double-back slashes when providing MS-DOS
#          paths.
#
#          This program is intended for use in MS-DOS/Windows
#          operating systems.  There has been some effort made to
#          handle file paths in a way that will accomodate use on
#          Macintosh or Linux OS, but that type of use has not 
#          been verified to function correctly via testing. 
#          This program therefore may not function correctly in
#          Unix, Linux, or Macintosh operating systems, although
#          it should be relatively easy to extend this program to
#          run correctly in those environments.  
#
# $RCSfile: zip_backup.pl,v $ 
# $Revision: 1.5 $ 
# $Date: 2003/07/08 22:13:41 $   
# $Author: Larry $
#  

# NOTE:    Backslashes used as path separators in MS-Dos/Windows 
#          have a special interpretation in Perl.  To ensure that
#          this program does not interpret slashes within a path
#          in that fashion, each backslash in a Dos/Windows path 
#          must be entered as TWO backslashes in the assignments
#          below
# EXAMPLE: 'E:\\Program Files\\WinZip\\'
# EXAMPLE: '\\\\hostpc\\shared_data\\'
#

# Specify operating system
# USER MODIFICATION
$os_string = "MSDOS";         #Valid values are 'MSDOS','MacOS','UNIX'

# Specify directory where WinZip is installed, with path separator at end
#   of string
# USER MODIFICATION
$wz_dir    = "C:\\Program Files\\WinZip\\";
print "wz_dir is '$wz_dir'";

# Specify source directory (loctn of files to be backed up), with path
#   separator at end of string
# USER MODIFICATION
$src_dir   = "d:\\work\\prof\\zip_backup_dev\\work\\src\\db\\";

# Specify naming pattern(s) for files to be backed up
#   Place within argument for 'qw' function 
#   If multiple patterns, separate with spaces and/or returns
# USER MODIFICATION
@src_pat_array = qw(*.102);
@src_pat_array = sort ( @src_pat_array );

# Create version of pattern specs used for display to user
$src_pat       = join ( " ", @src_pat_array);

# Specify destination directory for backup archive, with path separator at end
#   of string
# USER MODIFICATION
$des_dir   = "d:\\work\\prof\\zip_backup_dev\\work\\src\\db_archive\\";

# Specify base name pattern for archive files (date will be suffixed)
# USER MODIFICATION
$arch_name = "ebase_backup";

# Specify file (in archive directory) to which a record of the backup
#   will be written
# USER MODIFICATION
$history_file_name = "ebase_backup_history.txt"; 

# Notify user that program is running
print "\n\nProgram ZIP.BACKUP.PL started\n";
print "This program is designed to backup files to a WinZip archive\n\n";

# Assign appropriate path separator based on specified operating system
if ($os_string eq "MSDOS") {
    $path_separator = "\\";
}
elsif ($os_string eq "MacOS") { 
    $path_separator = "::";
}
else {
    $path_separator = "/";
}

#Correct directory specifications for common problems
$wz_dir  = clean_path($wz_dir);
$src_dir = clean_path($src_dir);
$des_dir = clean_path($des_dir);

# Print directory and file specs for user review
print "\n\nThe following specifications are being used\n\n";
print "   WinZip_dir:          ".$wz_dir           ."\n\n";
print "   Source dir:          ".$src_dir          ."\n";
print "   File name pattern:   ".$src_pat          ."\n\n";
print "   Target dir:          ".$des_dir          ."\n";
print "   Base archive name:   ".$arch_name        ."\n";
print "   Backup history file: ".$history_file_name."\n";

# Allow user to stop program without creating the backup
print "\n\nContinue? (<return>='yes', anything else = 'no')\n";
$user_response = <STDIN>;
chomp $user_response;
print "User response is '$user_response'\n";

if ($user_response or $user_response eq "0") {
    print "\nProgram stopped in response to user input\n\n";
} 
else {
    print "\nProgram creating backup in response to user input\n\n";

    # Construct string holding todays date
    ($min,$hr,$day,$mon,$yr) = (localtime)[1,2,3,4,5];
    $yr    += 1900;                #Perl returns yr minus 1900
    $mon   += 1;                   #Perl uses 0:11, add 1 to get "human" form
    if ($mon < 10) {
        $mon = "0" . $mon;         #All months provided using two digits
    }
    $day   += 0;                   #Perl returns expected day in month
    if ($day < 10) {
        $day = "0" . $day;         #All months provided using two digits
    }
    if ($hr  < 10) {
        $hr  = " " . $hr;          #All hrs provided using two characters
    }
    if ($min < 10) {
        $min = "0" . $min;         #All minutes provided using two digits
    }
    $tdate  = $yr . "_" . $mon . "_" . $day;
    $ttime  = $hr . ":" . $min;

    # Construct name for archive file
    $arch_name .= "_" . $tdate . '.zip';
    $arch_path  = dquote_val( $des_dir . $arch_name);

    # Construct base command for WinZip
    $wz_cmd           = dquote_val($wz_dir . 'wzzip') . ' -a ';
    $wz_cmd           = $wz_cmd . " " . $arch_path;

    foreach  $ind_file_pat (@src_pat_array) {
        $ind_file_path = dquote_val ($src_dir . $ind_file_pat);
        $ind_wz_cmd    = $wz_cmd . " " . $ind_file_path;
        $status = system("$ind_wz_cmd");
        if ($status) {
            print "\nERROR: $ind_file_path source file not added to archive\n";
            $overall_status = $status;
        }
    }

    # Let user know whether program finished without errors
    if ($overall_status) {
        print "\nERROR: Archiving failed on at least one source file,";
        print "\n       no message written to history file\n\n";
    }
    else {

        # If backup finished without errors, a description of the backup is
        # written to the backup history file
        print "\nBackup archive file created\n\n";
        print "   Archive file:      $arch_name\n\n";

        # Construct complete path to history file
        $history_file_path = $des_dir . $history_file_name;

        # Begin writing info about backup to backup history file
        unless ( open (HISTFILE, ">>$history_file_path")) {
            print "\nERROR: Specified history file NOT opened";
        } else {

            print HISTFILE "\n$tdate  $ttime        Archive file: $arch_name\n";

            $ind_file_lines_written = 0;
            foreach $ind_file_pat (@src_pat_array) {
                if ($ind_file_lines_written < 1) {
                    print HISTFILE "                  Directory backed up: $src_dir\n";
                    print HISTFILE "                      Files backed up: $ind_file_pat\n";
                    $ind_file_lines_written += 1;
                } else {
                    print HISTFILE "                                       $ind_file_pat\n";
                }
            }

            close (HISTFILE);

            print "Backup history record written\n\n";
            print "   History file:      $history_file_name\n";

        }
    }
}

# Wait for user response before closing window
#system("pause");

# Display window until user is done reviewing
print "\n\nClose window? (any response with a return = 'yes')\n";
$user_response = <STDIN>;

sub clean_path {

    # This subroutine accepts a text string representing a 
    #   directory path as input and returns a version of the path which:
    #     1. adds a trailing path separator when that is not present in the input
    #        path description
    #     2. removes quotes placed within the path
    #        (these may cause problems during later command construction
    #         steps which add quotes to the entire path).

    # Retrieve path supplied in subroutine call
    my $path = (@_[0]);

    # Remove any single or double quotes from path
    $path =~ s/['"]//g;

    # If the path separator is a backslash, add a backslash to prevent special
    #   interpretations during regexp use
    $separator_match = ($path_separator eq "\\") ? "\\$path_separator" : $path_separator;

    $path =~ s"([^${separator_match}])$"$1$path_separator";

    print "Matched string is '$1'\n";

    print "Clean_path: Final path is '$path'\n\n";

    return $path;

}

sub dquote_val {

    # This subroutine accepts a text string, checks it for spaces, 
    #   and if spaces are found, returns a value that
    #   encloses that string in double quotes.  

    # If no spaces are present, the subroutine returns the original
    #   value

    # Within the zip_backup program it is used to package path values
    #   that contain spaces - a function that is needed when 
    #   contructing commands to be passed to the archiving software

    #Retrieve value provided in subroutine call
    my $val = (@_[0]);

    if ( $val =~ / / ) {
        $qv = '"' . $val . '"';
    } else {
        $qv = $val;
    }

    return  $qv;

}

